@@ -17,6 +17,8 @@ import pecan
 from wsme import types as wtypes
 import wsmeext.pecan as wsme_pecan
 
+from mistral.api.controllers import info
+from mistral.api.controllers import maintenance
 from mistral.api.controllers import resource
 from mistral.api.controllers.v2 import root as v2_root
 
@@ -62,12 +64,14 @@ class APIVersions(resource.Resource):
 
 class RootController(object):
     v2 = v2_root.Controller()
+    info = info.InfoController()
+    maintenance = maintenance.MaintenanceController()
 
     @wsme_pecan.wsexpose(APIVersions)
     def index(self):
         LOG.debug("Fetching API versions.")
 
-        host_url_v2 = '%s/%s' % (pecan.request.host_url, 'v2')
+        host_url_v2 = '%s/%s' % (pecan.request.application_url, 'v2')
         api_v2 = APIVersion(
             id='v2.0',
             status='CURRENT',
