@@ -1,5 +1,6 @@
 # Copyright 2013 - Mirantis, Inc.
 # Copyright 2015 - StackStorm, Inc.
+# Copyright 2019 - NetCracker Technology Corp.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -18,14 +19,17 @@ import json
 from oslo_log import log as logging
 from pecan import rest
 from wsme import types as wtypes
+from wsme import Unset
 import wsmeext.pecan as wsme_pecan
 
 from mistral.api import access_control as acl
 from mistral.api.controllers.v2 import action_execution
 from mistral.api.controllers.v2 import resources
+from mistral.api.controllers.v2 import sub_execution
 from mistral.api.controllers.v2 import types
 from mistral import context
 from mistral.db.v2 import api as db_api
+from mistral.db.v2.sqlalchemy import models as db_models
 from mistral import exceptions as exc
 from mistral import expressions as expr
 from mistral.lang import parser as spec_parser
@@ -47,19 +51,22 @@ STATE_TYPES = wtypes.Enum(
 )
 
 
-def _get_task_resource_with_result(task_ex):
-    task = resources.Task.from_db_model(task_ex)
-
-    task.result = json.dumps(data_flow.get_task_execution_result(task_ex))
+def _get_task_resource_with_result(task_ex, fields=()):
+    task = resources.Task.from_db_model(task_ex, fields=fields)
+    if 'result' in fields or not fields:
+        task.result = json.dumps(data_flow.get_task_execution_result(task_ex))
 
     return task
 
 
 # Use retries to prevent possible failures.
 @rest_utils.rest_retry_on_db_error
-def _get_task_execution(id):
+def _get_task_execution(id, fields=()):
+    if fields and 'id' not in fields:
+        fields.insert(0, 'id')
+
     with db_api.transaction():
-        task_ex = db_api.get_task_execution(id)
+        task_ex = db_api.get_task_execution(id, fields=fields)
 
         rest_utils.load_deferred_fields(task_ex, ['workflow_execution'])
         rest_utils.load_deferred_fields(
@@ -72,7 +79,7 @@ def _get_task_execution(id):
             ['params']
         )
 
-        return _get_task_resource_with_result(task_ex), task_ex
+        return _get_task_resource_with_result(task_ex, fields), task_ex
 
 
 def get_published_global(task_ex, wf_ex=None):
@@ -114,14 +121,16 @@ class TaskExecutionsController(rest.RestController):
     @rest_utils.wrap_wsme_controller_exception
     @wsme_pecan.wsexpose(resources.Executions, types.uuid, types.uuid, int,
                          types.uniquelist, types.list, types.uniquelist,
-                         wtypes.text, types.uuid, wtypes.text, types.jsontype,
-                         STATE_TYPES, wtypes.text, types.jsontype,
-                         types.jsontype, wtypes.text, wtypes.text)
+                         wtypes.text, types.uuid, wtypes.text,
+                         types.uniquelist, types.jsontype, STATE_TYPES,
+                         wtypes.text, types.jsontype, types.jsontype,
+                         wtypes.text, wtypes.text)
     def get_all(self, task_execution_id, marker=None, limit=None,
                 sort_keys='created_at', sort_dirs='asc', fields='',
                 workflow_name=None, workflow_id=None, description=None,
-                params=None, state=None, state_info=None, input=None,
-                output=None, created_at=None, updated_at=None):
+                tags=None, params=None, state=None,
+                state_info=None, input=None, output=None,
+                created_at=None, updated_at=None):
         """Return all executions that belong to the given task execution.
 
         :param task_execution_id: Task task execution ID.
@@ -137,7 +146,7 @@ class TaskExecutionsController(rest.RestController):
                           or less than that of sort_keys.
         :param fields: Optional. A specified list of fields of the resource to
                        be returned. 'id' will be included automatically in
-                       fields if it's provided, since it will be used when
+                       fields if it's not provided, since it will be used when
                        constructing 'next' link.
         :param workflow_name: Optional. Keep only resources with a specific
                               workflow name.
@@ -145,6 +154,7 @@ class TaskExecutionsController(rest.RestController):
                             workflow ID.
         :param description: Optional. Keep only resources with a specific
                             description.
+        :param tags: Optional. Keep only resources containing specific tags.
         :param params: Optional. Keep only resources with specific parameters.
         :param state: Optional. Keep only resources with a specific state.
         :param state_info: Optional. Keep only resources with specific
@@ -163,6 +173,7 @@ class TaskExecutionsController(rest.RestController):
             created_at=created_at,
             workflow_name=workflow_name,
             workflow_id=workflow_id,
+            tags=tags,
             params=params,
             state=state,
             state_info=state_info,
@@ -195,32 +206,45 @@ class TaskExecutionsController(rest.RestController):
 class TasksController(rest.RestController):
     action_executions = action_execution.TasksActionExecutionController()
     workflow_executions = TaskExecutionsController()
+    executions = sub_execution.SubExecutionsController()
 
     @rest_utils.wrap_wsme_controller_exception
-    @wsme_pecan.wsexpose(resources.Task, wtypes.text)
-    def get(self, id):
+    @wsme_pecan.wsexpose(resources.Task, wtypes.text, types.uniquelist)
+    def get(self, id, fields=''):
         """Return the specified task.
 
         :param id: UUID of task to retrieve
+        :param fields: Optional. A specified list of fields of the resource to
+                       be returned. 'id' will be included automatically in
+                       fields if it's not provided.
         """
         acl.enforce('tasks:get', context.ctx())
         LOG.debug("Fetch task [id=%s]", id)
 
-        task, task_ex = _get_task_execution(id)
-
-        return _task_with_published_global(task, task_ex)
+        task, task_ex = _get_task_execution(id, ())
+        task = _task_with_published_global(task, task_ex)
+        if fields:
+            if 'id' not in fields:
+                fields.insert(0, 'id')
+            task_dict = {field: task.to_dict()[field] for field in fields}
+            task = resources.Task.from_dict(task_dict)
+        return task
 
     @rest_utils.wrap_wsme_controller_exception
     @wsme_pecan.wsexpose(resources.Tasks, types.uuid, int, types.uniquelist,
                          types.list, types.uniquelist, wtypes.text,
-                         wtypes.text, types.uuid, types.uuid, STATE_TYPES,
-                         wtypes.text, wtypes.text, types.jsontype, bool,
-                         wtypes.text, wtypes.text, bool, types.jsontype)
+                         wtypes.text, types.uuid,
+                         types.uuid, types.uniquelist, STATE_TYPES,
+                         wtypes.text, wtypes.text, types.jsontype,
+                         bool, wtypes.text, wtypes.text,
+                         bool, types.jsontype)
     def get_all(self, marker=None, limit=None, sort_keys='created_at',
-                sort_dirs='asc', fields='', name=None, workflow_name=None,
-                workflow_id=None, workflow_execution_id=None, state=None,
-                state_info=None, result=None, published=None, processed=None,
-                created_at=None, updated_at=None, reset=None, env=None):
+                sort_dirs='asc', fields='', name=None,
+                workflow_name=None, workflow_id=None,
+                workflow_execution_id=None, tags=None, state=None,
+                state_info=None, result=None, published=None,
+                processed=None, created_at=None, updated_at=None,
+                reset=None, env=None):
         """Return all tasks.
 
         Where project_id is the same as the requester or
@@ -238,7 +262,7 @@ class TasksController(rest.RestController):
                           or less than that of sort_keys.
         :param fields: Optional. A specified list of fields of the resource to
                        be returned. 'id' will be included automatically in
-                       fields if it's provided, since it will be used when
+                       fields if it's not provided, since it will be used when
                        constructing 'next' link.
         :param name: Optional. Keep only resources with a specific name.
         :param workflow_name: Optional. Keep only resources with a specific
@@ -269,6 +293,7 @@ class TasksController(rest.RestController):
             created_at=created_at,
             workflow_name=workflow_name,
             workflow_id=workflow_id,
+            tags=tags,
             state=state,
             state_info=state_info,
             updated_at=updated_at,
@@ -309,7 +334,7 @@ class TasksController(rest.RestController):
         """
         acl.enforce('tasks:update', context.ctx())
 
-        LOG.debug("Update task execution [id=%s, task=%s]", id, task)
+        LOG.info("Update task execution [id=%s, task=%s]", id, task)
 
         @rest_utils.rest_retry_on_db_error
         def _read_task_params(id, task):
@@ -317,7 +342,7 @@ class TasksController(rest.RestController):
                 task_ex = db_api.get_task_execution(id)
                 task_spec = spec_parser.get_task_spec(task_ex.spec)
                 task_name = task.name or None
-                reset = task.reset
+                reset = task.reset or None
                 env = task.env or None
 
                 if task_name and task_name != task_ex.name:
@@ -327,6 +352,12 @@ class TasksController(rest.RestController):
                     task_ex.workflow_execution_id
                 )
 
+                root_execution_id = wf_ex.root_execution_id
+                if not root_execution_id:
+                    root_execution_id = wf_ex.id
+
+                context.ctx(root_execution_id=root_execution_id)
+
                 return env, reset, task_ex, task_spec, wf_ex
 
         env, reset, task_ex, task_spec, wf_ex = _read_task_params(id, task)
@@ -336,10 +367,10 @@ class TasksController(rest.RestController):
         if wf_name and wf_name != wf_ex.name:
             raise exc.WorkflowException('Workflow name does not match.')
 
-        if task.state != states.RUNNING:
+        if task.state != states.RUNNING and task.state != states.SKIPPED:
             raise exc.WorkflowException(
                 'Invalid task state. '
-                'Only updating task to rerun is supported.'
+                'Only updating task to RUNNING or SKIPPED is supported.'
             )
 
         if task_ex.state != states.ERROR:
@@ -348,14 +379,21 @@ class TasksController(rest.RestController):
                 ' Only updating task to rerun is supported.'
             )
 
-        if not task_spec.get_with_items() and not reset:
-            raise exc.WorkflowException(
-                'Only with-items task has the option to not reset.'
-            )
+        if task.state == states.RUNNING:
+            if task.reset is Unset:
+                raise exc.WorkflowException(
+                    'Reset field is mandatory to rerun task.'
+                )
+
+            if not task_spec.get_with_items() and not reset:
+                raise exc.WorkflowException(
+                    'Only with-items task has the option to not reset.'
+                )
 
         rpc.get_engine_client().rerun_workflow(
             task_ex.id,
             reset=reset,
+            skip=(task.state == states.SKIPPED),
             env=env
         )
 
@@ -373,13 +411,15 @@ class ExecutionTasksController(rest.RestController):
     @rest_utils.wrap_wsme_controller_exception
     @wsme_pecan.wsexpose(resources.Tasks, types.uuid, types.uuid, int,
                          types.uniquelist, types.list, types.uniquelist,
-                         wtypes.text, wtypes.text, types.uuid, STATE_TYPES,
-                         wtypes.text, wtypes.text, types.jsontype, bool,
+                         wtypes.text, wtypes.text, types.uuid,
+                         types.uniquelist, STATE_TYPES, wtypes.text,
+                         wtypes.text, types.jsontype, bool,
                          wtypes.text, wtypes.text, bool, types.jsontype)
     def get_all(self, workflow_execution_id, marker=None, limit=None,
-                sort_keys='created_at', sort_dirs='asc', fields='', name=None,
-                workflow_name=None, workflow_id=None, state=None,
-                state_info=None, result=None, published=None, processed=None,
+                sort_keys='created_at', sort_dirs='asc', fields='',
+                name=None, workflow_name=None, workflow_id=None,
+                tags=None, state=None, state_info=None,
+                result=None, published=None, processed=None,
                 created_at=None, updated_at=None, reset=None, env=None):
         """Return all tasks within the execution.
 
@@ -398,7 +438,7 @@ class ExecutionTasksController(rest.RestController):
                           or less than that of sort_keys.
         :param fields: Optional. A specified list of fields of the resource to
                        be returned. 'id' will be included automatically in
-                       fields if it's provided, since it will be used when
+                       fields if it's not provided, since it will be used when
                        constructing 'next' link.
         :param name: Optional. Keep only resources with a specific name.
         :param workflow_name: Optional. Keep only resources with a specific
@@ -407,6 +447,7 @@ class ExecutionTasksController(rest.RestController):
                             workflow ID.
         :param workflow_execution_id: Optional. Keep only resources with a
                                       specific workflow execution ID.
+        :param tags: Optional. Keep only resources containing specific tags.
         :param state: Optional. Keep only resources with a specific state.
         :param state_info: Optional. Keep only resources with specific
                            state information.
@@ -425,11 +466,18 @@ class ExecutionTasksController(rest.RestController):
         """
         acl.enforce('tasks:list', context.ctx())
 
+        with db_api.transaction():
+            db_api.get_workflow_execution(
+                workflow_execution_id,
+                fields=(db_models.WorkflowExecution.id,)
+            )
+
         filters = filter_utils.create_filters_from_request_params(
             workflow_execution_id=workflow_execution_id,
             created_at=created_at,
             workflow_name=workflow_name,
             workflow_id=workflow_id,
+            tags=tags,
             state=state,
             state_info=state_info,
             updated_at=updated_at,
