@@ -13,8 +13,8 @@
 #    limitations under the License.
 import json
 
+from mistral.utils import filter_utils
 from oslo_utils import uuidutils
-import six
 from wsme import types as wtypes
 
 from mistral import exceptions as exc
@@ -33,7 +33,7 @@ class ListType(wtypes.UserType):
         :param value: A comma separated string of values
         :returns: A list of values.
         """
-        items = [v.strip().lower() for v in six.text_type(value).split(',')]
+        items = [v.strip().lower() for v in str(value).split(',')]
 
         # remove empty items.
         return [x for x in items if x]
@@ -77,12 +77,13 @@ class UuidType(wtypes.UserType):
 
     @staticmethod
     def validate(value):
-        if not uuidutils.is_uuid_like(value):
+        _, data = filter_utils.extract_filter_type_and_value(value)
+        if not uuidutils.is_uuid_like(data):
             raise exc.InputException(
-                "Expected a uuid but received %s." % value
+                "Expected a uuid but received %s." % data
             )
 
-        return value
+        return data
 
     @staticmethod
     def frombasetype(value):
