@@ -19,6 +19,7 @@ from mistral_lib.actions import context as lib_ctx
 from mistral_lib import serialization
 from oslo_config import cfg
 from oslo_context import context as oslo_context
+from oslo_log import log as logging
 import oslo_messaging as messaging
 from oslo_serialization import jsonutils
 from osprofiler import profiler
@@ -29,16 +30,29 @@ from mistral import auth
 from mistral import exceptions as exc
 from mistral_lib import utils
 
+
+LOG = logging.getLogger(__name__)
+
 CONF = cfg.CONF
+
 _CTX_THREAD_LOCAL_NAME = "MISTRAL_APP_CTX_THREAD_LOCAL"
-ALLOWED_WITHOUT_AUTH = ['/', '/v2/', '/workflowv2/', '/workflowv2/v2/']
+ALLOWED_WITHOUT_AUTH = [
+    '/',
+    '/info',
+    '/v2/',
+    '/workflowv2/',
+    '/workflowv2/v2/'
+]
 
 
 class MistralContext(oslo_context.RequestContext):
+
+    FROM_DICT_EXTRA_KEYS = ['root_execution_id']
+
     def __init__(self, auth_uri=None, auth_cacert=None, insecure=False,
                  service_catalog=None, region_name=None, is_trust_scoped=False,
                  redelivered=False, expires_at=None, trust_id=None,
-                 is_target=False, **kwargs):
+                 is_target=False, root_execution_id=None, **kwargs):
         self.auth_uri = auth_uri
         self.auth_cacert = auth_cacert
         self.insecure = insecure
@@ -49,11 +63,25 @@ class MistralContext(oslo_context.RequestContext):
         self.expires_at = expires_at
         self.trust_id = trust_id
         self.is_target = is_target
+        self.root_execution_id = root_execution_id
 
         # We still use Mistral thread local variable. Maybe could consider
         # using the variable provided by oslo_context in future.
         super(MistralContext, self).__init__(overwrite=False, **kwargs)
 
+    def get_logging_values(self):
+        ctx_dict = super(MistralContext, self).get_logging_values()
+        ctx_dict.update(
+            {
+                'root_execution_id': self.root_execution_id,
+            }
+        )
+
+        return ctx_dict
+
+    def update_store(self):
+        super(MistralContext, self).update_store()
+
     def to_dict(self):
         """Return a dictionary of context attributes."""
         ctx_dict = super(MistralContext, self).to_dict()
@@ -74,6 +102,7 @@ class MistralContext(oslo_context.RequestContext):
                 'expires_at': self.expires_at,
                 'trust_id': self.trust_id,
                 'is_target': self.is_target,
+                'root_execution_id': self.root_execution_id,
             }
         )
 
@@ -94,6 +123,7 @@ class MistralContext(oslo_context.RequestContext):
         kwargs.setdefault('expires_at', values.get('expires_at'))
         kwargs.setdefault('trust_id', values.get('trust_id'))
         kwargs.setdefault('is_target', values.get('is_target', False))
+        kwargs.setdefault('root_execution_id', values.get('root_execution_id'))
 
         return super(MistralContext, cls).from_dict(values, **kwargs)
 
@@ -117,14 +147,22 @@ def has_ctx():
     return utils.has_thread_local(_CTX_THREAD_LOCAL_NAME)
 
 
-def ctx():
+def ctx(root_execution_id=None):
     if not has_ctx():
         raise exc.ApplicationContextNotFoundException()
 
-    return utils.get_thread_local(_CTX_THREAD_LOCAL_NAME)
+    context = utils.get_thread_local(_CTX_THREAD_LOCAL_NAME)
+
+    if root_execution_id:
+        context.root_execution_id = root_execution_id
+        set_ctx(context)
+
+    return context
 
 
 def set_ctx(new_ctx):
+    if new_ctx:
+        new_ctx.update_store()
     utils.set_thread_local(_CTX_THREAD_LOCAL_NAME, new_ctx)
 
 
@@ -148,7 +186,7 @@ def _extract_mistral_auth_params(headers):
             'insecure': insecure,
             'auth_token': headers.get('X-Target-Auth-Token'),
             'auth_uri': headers.get('X-Target-Auth-Uri'),
-            'tenant': headers.get('X-Target-Project-Id'),
+            'project_id': headers.get('X-Target-Project-Id'),
             'user': headers.get('X-Target-User-Id'),
             'user_name': headers.get('X-Target-User-Name'),
             'region_name': headers.get('X-Target-Region-Name'),
@@ -249,6 +287,8 @@ class AuthHook(hooks.PecanHook):
         except Exception as e:
             msg = "Failed to validate access token: %s" % str(e)
 
+            LOG.exception(msg)
+
             pecan.abort(
                 status_code=401,
                 detail=msg,
