@@ -21,10 +21,8 @@ from alembic import util as alembic_u
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import importutils
-import six
 import sys
 
-from mistral import config
 from mistral.services import action_manager
 from mistral.services import workflows
 
@@ -42,7 +40,7 @@ def do_alembic_command(config, cmd, *args, **kwargs):
     try:
         getattr(alembic_cmd, cmd)(config, *args, **kwargs)
     except alembic_u.CommandError as e:
-        alembic_u.err(six.text_type(e))
+        alembic_u.err(str(e))
 
 
 def do_check_migration(config, _cmd):
@@ -76,6 +74,11 @@ def do_populate(config, cmd):
     workflows.sync_db()
 
 
+def do_populate_actions(config, cmd):
+    LOG.info("Populating actions db")
+    action_manager.sync_db()
+
+
 def do_revision(config, cmd):
     do_alembic_command(
         config, cmd,
@@ -100,6 +103,9 @@ def add_command_parsers(subparsers):
     parser = subparsers.add_parser('populate')
     parser.set_defaults(func=do_populate)
 
+    parser = subparsers.add_parser('populate_actions')
+    parser.set_defaults(func=do_populate_actions)
+
     parser = subparsers.add_parser('stamp')
     parser.add_argument('--sql', action='store_true')
     parser.add_argument('revision', nargs='?')
@@ -119,7 +125,14 @@ command_opt = cfg.SubCommandOpt('command',
                                 handler=add_command_parsers)
 
 CONF.register_cli_opt(command_opt)
-CONF.register_cli_opt(config.os_actions_mapping_path)
+
+# To Keep backwards compatibility we need to accept mapping path
+# from mistral-extra if present
+try:
+    import mistral_extra.config as extra_conf
+    CONF.register_cli_opt(extra_conf.os_actions_mapping_path)
+except ImportError:
+    LOG.debug("Mistral-extra not installed")
 
 
 def main():
