@@ -24,7 +24,6 @@ from mistral.utils import wf_trace
 from mistral.workflow import data_flow
 from mistral.workflow import states
 
-import six
 
 _CONTINUE_TASK_PATH = 'mistral.engine.policies._continue_task'
 _COMPLETE_TASK_PATH = 'mistral.engine.policies._complete_task'
@@ -33,14 +32,6 @@ _FAIL_IF_INCOMPLETE_TASK_PATH = (
 )
 
 
-def _log_task_delay(task_ex, delay_sec, state=states.RUNNING_DELAYED):
-    wf_trace.info(
-        task_ex,
-        "Task '%s' [%s -> %s, delay = %s sec]" %
-        (task_ex.name, task_ex.state, state, delay_sec)
-    )
-
-
 def build_policies(policies_spec, wf_spec):
     task_defaults = wf_spec.get_task_defaults()
     wf_policies = task_defaults.get_policies() if task_defaults else None
@@ -84,7 +75,7 @@ def build_wait_before_policy(policies_spec):
 
     wait_before = policies_spec.get_wait_before()
 
-    if isinstance(wait_before, six.string_types) or wait_before > 0:
+    if isinstance(wait_before, str) or wait_before > 0:
         return WaitBeforePolicy(wait_before)
     else:
         return None
@@ -96,7 +87,7 @@ def build_wait_after_policy(policies_spec):
 
     wait_after = policies_spec.get_wait_after()
 
-    if isinstance(wait_after, six.string_types) or wait_after > 0:
+    if isinstance(wait_after, str) or wait_after > 0:
         return WaitAfterPolicy(wait_after)
     else:
         return None
@@ -125,7 +116,7 @@ def build_timeout_policy(policies_spec):
 
     timeout_policy = policies_spec.get_timeout()
 
-    if isinstance(timeout_policy, six.string_types) or timeout_policy > 0:
+    if isinstance(timeout_policy, str) or timeout_policy > 0:
         return TimeoutPolicy(timeout_policy)
     else:
         return None
@@ -184,51 +175,37 @@ class WaitBeforePolicy(base.TaskPolicy):
     def __init__(self, delay):
         self.delay = delay
 
-    def before_task_start(self, task_ex, task_spec):
-        super(WaitBeforePolicy, self).before_task_start(task_ex, task_spec)
+    def before_task_start(self, task):
+        super(WaitBeforePolicy, self).before_task_start(task)
 
         # No need to wait for a task if delay is 0
         if self.delay == 0:
             return
 
-        context_key = 'wait_before_policy'
+        ctx_key = 'wait_before_policy'
 
-        runtime_context = _ensure_context_has_key(
-            task_ex.runtime_context,
-            context_key
-        )
+        policy_ctx = task.get_policy_context(ctx_key)
 
-        task_ex.runtime_context = runtime_context
-
-        policy_context = runtime_context[context_key]
-
-        if policy_context.get('skip'):
+        if policy_ctx.get('skip'):
             # Unset state 'RUNNING_DELAYED'.
-            wf_trace.info(
-                task_ex,
-                "Task '%s' [%s -> %s]"
-                % (task_ex.name, states.RUNNING_DELAYED, states.RUNNING)
-            )
-
-            task_ex.state = states.RUNNING
+            task.set_state(states.RUNNING, None)
 
             return
 
-        if task_ex.state != states.IDLE:
-            policy_context.update({'skip': True})
+        if task.get_state() != states.IDLE:
+            policy_ctx.update({'skip': True})
 
-            _log_task_delay(task_ex, self.delay)
-
-            task_ex.state = states.RUNNING_DELAYED
+            task.set_state(
+                states.RUNNING_DELAYED,
+                "Delayed by 'wait-before' policy [delay=%s]" % self.delay
+            )
 
             sched = sched_base.get_system_scheduler()
 
             job = sched_base.SchedulerJob(
                 run_after=self.delay,
                 func_name=_CONTINUE_TASK_PATH,
-                func_args={
-                    'task_ex_id': task_ex.id
-                }
+                func_args={'task_ex_id': task.get_id()}
             )
 
             sched.schedule(job)
@@ -247,41 +224,30 @@ class WaitAfterPolicy(base.TaskPolicy):
     def __init__(self, delay):
         self.delay = delay
 
-    def after_task_complete(self, task_ex, task_spec):
-        super(WaitAfterPolicy, self).after_task_complete(task_ex, task_spec)
+    def after_task_complete(self, task):
+        super(WaitAfterPolicy, self).after_task_complete(task)
 
         # No need to postpone a task if delay is 0
         if self.delay == 0:
             return
 
-        context_key = 'wait_after_policy'
+        ctx_key = 'wait_after_policy'
 
-        runtime_context = _ensure_context_has_key(
-            task_ex.runtime_context,
-            context_key
-        )
+        policy_ctx = task.get_policy_context(ctx_key)
 
-        task_ex.runtime_context = runtime_context
-
-        policy_context = runtime_context[context_key]
-
-        if policy_context.get('skip'):
+        if policy_ctx.get('skip'):
             # Skip, already processed.
             return
 
-        policy_context.update({'skip': True})
+        policy_ctx.update({'skip': True})
 
-        _log_task_delay(task_ex, self.delay)
+        end_state = task.get_state()
+        end_state_info = task.get_state_info()
 
-        end_state = task_ex.state
-        end_state_info = task_ex.state_info
-
-        # TODO(rakhmerov): Policies probably need to have tasks.Task
-        # interface in order to manage task state safely.
         # Set task state to 'RUNNING_DELAYED'.
-        task_ex.state = states.RUNNING_DELAYED
-        task_ex.state_info = (
-            'Suspended by wait-after policy for %s seconds' % self.delay
+        task.set_state(
+            states.RUNNING_DELAYED,
+            "Delayed by 'wait-after' policy [delay=%s]" % self.delay
         )
 
         # Schedule to change task state to RUNNING again.
@@ -291,7 +257,7 @@ class WaitAfterPolicy(base.TaskPolicy):
             run_after=self.delay,
             func_name=_COMPLETE_TASK_PATH,
             func_args={
-                'task_ex_id': task_ex.id,
+                'task_ex_id': task.get_id(),
                 'state': end_state,
                 'state_info': end_state_info
             }
@@ -320,7 +286,7 @@ class RetryPolicy(base.TaskPolicy):
         self._break_on_clause = break_on
         self._continue_on_clause = continue_on
 
-    def after_task_complete(self, task_ex, task_spec):
+    def after_task_complete(self, task):
         """Possible Cases:
 
         1. state = SUCCESS
@@ -336,7 +302,7 @@ class RetryPolicy(base.TaskPolicy):
         3. retry:count = 5, current:count = 4, state = ERROR
         Iterations complete therefore state = #{state}, current:count = 4.
         """
-        super(RetryPolicy, self).after_task_complete(task_ex, task_spec)
+        super(RetryPolicy, self).after_task_complete(task)
 
         # There is nothing to repeat
         if self.count == 0:
@@ -347,53 +313,42 @@ class RetryPolicy(base.TaskPolicy):
         # then the retry_no in the runtime_context of the task_ex will not
         # be updated accurately. To be exact, the retry_no will be one
         # iteration behind.
-        ex = task_ex.executions  # noqa
+        ex = task.task_ex.executions  # noqa
 
-        context_key = 'retry_task_policy'
+        ctx_key = 'retry_task_policy'
 
-        runtime_context = _ensure_context_has_key(
-            task_ex.runtime_context,
-            context_key
-        )
-
-        wf_ex = task_ex.workflow_execution
-
-        ctx_view = data_flow.ContextView(
-            data_flow.get_current_task_dict(task_ex),
-            data_flow.evaluate_task_outbound_context(task_ex),
-            wf_ex.context,
-            wf_ex.input
+        expr_ctx = task.get_expression_context(
+            ctx=data_flow.evaluate_task_outbound_context(task.task_ex)
         )
 
         continue_on_evaluation = expressions.evaluate(
             self._continue_on_clause,
-            ctx_view
+            expr_ctx
         )
 
         break_on_evaluation = expressions.evaluate(
             self._break_on_clause,
-            ctx_view
+            expr_ctx
         )
 
-        task_ex.runtime_context = runtime_context
-
-        state = task_ex.state
+        state = task.get_state()
 
         if not states.is_completed(state) or states.is_cancelled(state):
             return
 
-        policy_context = runtime_context[context_key]
+        policy_ctx = task.get_policy_context(ctx_key)
 
         retry_no = 0
 
-        if 'retry_no' in policy_context:
-            retry_no = policy_context['retry_no']
-            del policy_context['retry_no']
+        if 'retry_no' in policy_ctx:
+            retry_no = policy_ctx['retry_no']
+
+            del policy_ctx['retry_no']
 
         retries_remain = retry_no < self.count
 
         stop_continue_flag = (
-            task_ex.state == states.SUCCESS and
+            task.get_state() == states.SUCCESS and
             not self._continue_on_clause
         )
 
@@ -403,55 +358,53 @@ class RetryPolicy(base.TaskPolicy):
         )
 
         break_triggered = (
-            task_ex.state == states.ERROR and
+            task.get_state() == states.ERROR and
             break_on_evaluation
         )
 
         if not retries_remain or break_triggered or stop_continue_flag:
             return
 
-        data_flow.invalidate_task_execution_result(task_ex)
+        task.invalidate_result()
 
-        policy_context['retry_no'] = retry_no + 1
-        runtime_context[context_key] = policy_context
+        policy_ctx['retry_no'] = retry_no + 1
+
+        task.touch_runtime_context()
 
         # NOTE(vgvoleg): join tasks in direct workflows can't be
         # retried as-is, because these tasks can't start without
         # a correct logical state.
-        if hasattr(task_spec, "get_join") and task_spec.get_join():
+        if hasattr(task.task_spec, "get_join") and task.task_spec.get_join():
+            # TODO(rakhmerov): This is an example of broken encapsulation.
+            # The control over such operations should belong to the class Task.
+            # If it's done, from the outside of the class there will be just
+            # one visible operation "continue_task()" or something like that.
             from mistral.engine import task_handler as t_h
 
-            _log_task_delay(task_ex, self.delay, states.WAITING)
+            task.set_state(
+                states.WAITING,
+                "Delayed by 'retry' policy [delay=%s]" % self.delay
+            )
 
-            task_ex.state = states.WAITING
-
-            t_h._schedule_refresh_task_state(task_ex.id, self.delay)
+            t_h._schedule_refresh_task_state(task.get_id(), self.delay)
 
             return
 
-        _log_task_delay(task_ex, self.delay)
-
-        task_ex.state = states.RUNNING_DELAYED
+        task.set_state(
+            states.RUNNING_DELAYED,
+            "Delayed by 'retry' policy [delay=%s]" % self.delay
+        )
 
         sched = sched_base.get_system_scheduler()
 
         job = sched_base.SchedulerJob(
             run_after=self.delay,
             func_name=_CONTINUE_TASK_PATH,
-            func_args={'task_ex_id': task_ex.id}
+            func_args={'task_ex_id': task.get_id()}
         )
 
         sched.schedule(job)
 
-    @staticmethod
-    def refresh_runtime_context(task_ex):
-        runtime_context = task_ex.runtime_context or {}
-        retry_task_policy = runtime_context.get('retry_task_policy')
-
-        if retry_task_policy:
-            retry_task_policy['retry_no'] = 0
-            task_ex.runtime_context['retry_task_policy'] = retry_task_policy
-
 
 class TimeoutPolicy(base.TaskPolicy):
     _schema = {
@@ -466,8 +419,8 @@ class TimeoutPolicy(base.TaskPolicy):
     def __init__(self, timeout_sec):
         self.delay = timeout_sec
 
-    def before_task_start(self, task_ex, task_spec):
-        super(TimeoutPolicy, self).before_task_start(task_ex, task_spec)
+    def before_task_start(self, task):
+        super(TimeoutPolicy, self).before_task_start(task)
 
         # No timeout if delay is 0
         if self.delay == 0:
@@ -479,7 +432,7 @@ class TimeoutPolicy(base.TaskPolicy):
             run_after=self.delay,
             func_name=_FAIL_IF_INCOMPLETE_TASK_PATH,
             func_args={
-                'task_ex_id': task_ex.id,
+                'task_ex_id': task.get_id(),
                 'timeout': self.delay
             }
         )
@@ -487,9 +440,9 @@ class TimeoutPolicy(base.TaskPolicy):
         sched.schedule(job)
 
         wf_trace.info(
-            task_ex,
+            task.task_ex,
             "Timeout check scheduled [task=%s, timeout(s)=%s]." %
-            (task_ex.id, self.delay)
+            (task.get_id(), self.delay)
         )
 
 
@@ -503,20 +456,25 @@ class PauseBeforePolicy(base.TaskPolicy):
     def __init__(self, expression):
         self.expr = expression
 
-    def before_task_start(self, task_ex, task_spec):
-        super(PauseBeforePolicy, self).before_task_start(task_ex, task_spec)
+    def before_task_start(self, task):
+        super(PauseBeforePolicy, self).before_task_start(task)
 
         if not self.expr:
             return
 
         wf_trace.info(
-            task_ex,
+            task.task_ex,
             "Workflow paused before task '%s' [%s -> %s]" %
-            (task_ex.name, task_ex.workflow_execution.state, states.PAUSED)
+            (
+                task.get_name(),
+                task.wf_ex.state,
+                states.PAUSED
+            )
         )
 
-        task_ex.state = states.IDLE
-        wf_handler.pause_workflow(task_ex.workflow_execution)
+        task.set_state(states.IDLE, "Set by 'pause-before' policy")
+
+        wf_handler.pause_workflow(task.wf_ex)
 
 
 class ConcurrencyPolicy(base.TaskPolicy):
@@ -532,8 +490,8 @@ class ConcurrencyPolicy(base.TaskPolicy):
     def __init__(self, concurrency):
         self.concurrency = concurrency
 
-    def before_task_start(self, task_ex, task_spec):
-        super(ConcurrencyPolicy, self).before_task_start(task_ex, task_spec)
+    def before_task_start(self, task):
+        super(ConcurrencyPolicy, self).before_task_start(task)
 
         if self.concurrency == 0:
             return
@@ -542,15 +500,9 @@ class ConcurrencyPolicy(base.TaskPolicy):
         # property value and setting a variable into task runtime context.
         # This variable is then used to define how many action executions
         # may be started in parallel.
-        context_key = 'concurrency'
+        ctx_key = 'concurrency'
 
-        runtime_context = _ensure_context_has_key(
-            task_ex.runtime_context,
-            context_key
-        )
-
-        runtime_context[context_key] = self.concurrency
-        task_ex.runtime_context = runtime_context
+        task.set_runtime_context_value(ctx_key, self.concurrency)
 
 
 class FailOnPolicy(base.TaskPolicy):
@@ -563,18 +515,17 @@ class FailOnPolicy(base.TaskPolicy):
     def __init__(self, fail_on):
         self.fail_on = fail_on
 
-    def before_task_start(self, task_ex, task_spec):
+    def before_task_start(self, task):
         pass
 
-    def after_task_complete(self, task_ex, task_spec):
-        if task_ex.state != states.SUCCESS:
+    def after_task_complete(self, task):
+        super(FailOnPolicy, self).after_task_complete(task)
+
+        if task.get_state() != states.SUCCESS:
             return
 
-        super(FailOnPolicy, self).after_task_complete(task_ex, task_spec)
-
         if self.fail_on:
-            task_ex.state = states.ERROR
-            task_ex.state_info = 'Failed by fail-on policy'
+            task.set_state(states.ERROR, "Failed by 'fail-on' policy")
 
 
 @db_utils.retry_on_db_error
