@@ -17,7 +17,6 @@
 import copy
 import json
 import re
-import six
 
 from mistral import exceptions as exc
 from mistral import expressions
@@ -30,7 +29,13 @@ from mistral.lang.v2 import retry_policy
 from mistral.workflow import states
 from mistral_lib import utils
 
+# Task types.
+ACTION_TASK_TYPE = 'ACTION'
+WORKFLOW_TASK_TYPE = 'WORKFLOW'
+
+
 _expr_ptrns = [expressions.patterns[name] for name in expressions.patterns]
+
 WITH_ITEMS_PTRN = re.compile(
     r"\s*([\w\d_\-]+)\s*in\s*(\[.+\]|%s)" % '|'.join(_expr_ptrns)
 )
@@ -70,6 +75,7 @@ class TaskSpec(base.BaseSpec):
             },
             "publish": types.NONEMPTY_DICT,
             "publish-on-error": types.NONEMPTY_DICT,
+            "publish-on-skip": types.NONEMPTY_DICT,
             "retry": retry_policy.RetrySpec.get_schema(),
             "wait-before": types.EXPRESSION_OR_POSITIVE_INTEGER,
             "wait-after": types.EXPRESSION_OR_POSITIVE_INTEGER,
@@ -116,6 +122,7 @@ class TaskSpec(base.BaseSpec):
         self._with_items = self._get_with_items_as_dict()
         self._publish = data.get('publish', {})
         self._publish_on_error = data.get('publish-on-error', {})
+        self._publish_on_skip = data.get('publish-on-skip', {})
         self._policies = self._group_spec(
             policies.PoliciesSpec,
             'retry',
@@ -148,6 +155,7 @@ class TaskSpec(base.BaseSpec):
         self.validate_expr(self._data.get('input', {}))
         self.validate_expr(self._data.get('publish', {}))
         self.validate_expr(self._data.get('publish-on-error', {}))
+        self.validate_expr(self._data.get('publish-on-skip', {}))
         self.validate_expr(self._data.get('keep-result', {}))
         self.validate_expr(self._data.get('safe-rerun', {}))
 
@@ -164,11 +172,11 @@ class TaskSpec(base.BaseSpec):
 
         with_items = {}
 
-        if isinstance(raw, six.string_types):
+        if isinstance(raw, str):
             raw = [raw]
 
         for item in raw:
-            if not isinstance(item, six.string_types):
+            if not isinstance(item, str):
                 raise exc.InvalidModelException(
                     "'with-items' elements should be strings: %s" % self._data
                 )
@@ -255,6 +263,11 @@ class TaskSpec(base.BaseSpec):
                 {'branch': self._publish_on_error},
                 validate=self._validate
             )
+        elif state == states.SKIPPED and self._publish_on_skip:
+            spec = publish.PublishSpec(
+                {'branch': self._publish_on_skip},
+                validate=self._validate
+            )
         return spec
 
     def get_keep_result(self):
@@ -264,8 +277,8 @@ class TaskSpec(base.BaseSpec):
         return self._safe_rerun
 
     def get_type(self):
-        return (utils.WORKFLOW_TASK_TYPE if self._workflow
-                else utils.ACTION_TASK_TYPE)
+        return (WORKFLOW_TASK_TYPE if self._workflow
+                else ACTION_TASK_TYPE)
 
 
 class DirectWorkflowTaskSpec(TaskSpec):
@@ -283,7 +296,8 @@ class DirectWorkflowTaskSpec(TaskSpec):
             },
             "on-complete": on_clause.OnClauseSpec.get_schema(),
             "on-success": on_clause.OnClauseSpec.get_schema(),
-            "on-error": on_clause.OnClauseSpec.get_schema()
+            "on-error": on_clause.OnClauseSpec.get_schema(),
+            "on-skip": on_clause.OnClauseSpec.get_schema()
         }
     }
 
@@ -302,15 +316,18 @@ class DirectWorkflowTaskSpec(TaskSpec):
         self._on_complete = self._spec_property('on-complete', on_spec_cls)
         self._on_success = self._spec_property('on-success', on_spec_cls)
         self._on_error = self._spec_property('on-error', on_spec_cls)
+        self._on_skip = self._spec_property('on-skip', on_spec_cls)
 
     def validate_semantics(self):
         # Validate YAQL expressions.
         self._validate_transitions(self._on_complete)
         self._validate_transitions(self._on_success)
         self._validate_transitions(self._on_error)
+        self._validate_transitions(self._on_skip)
 
         if self._join:
             join_task_name = self.get_name()
+
             if len(join_task_name) > MAX_LENGTH_JOIN_TASK_NAME:
                 raise exc.InvalidModelException(
                     "The length of a '{0}' join task name must not exceed {1} "
@@ -324,7 +341,7 @@ class DirectWorkflowTaskSpec(TaskSpec):
             return
 
         [self.validate_expr(t)
-            for t in ([val] if isinstance(val, six.string_types) else val)]
+            for t in ([val] if isinstance(val, str) else val)]
 
     def get_publish(self, state):
         spec = super(DirectWorkflowTaskSpec, self).get_publish(state)
@@ -339,10 +356,13 @@ class DirectWorkflowTaskSpec(TaskSpec):
             on_clause = self._on_success
         elif state == states.ERROR:
             on_clause = self._on_error
+        elif state == states.SKIPPED:
+            on_clause = self._on_skip
 
         if on_clause and on_clause.get_publish():
             if spec:
                 on_clause.get_publish().merge(spec)
+
             return on_clause.get_publish()
 
         return spec
@@ -359,6 +379,9 @@ class DirectWorkflowTaskSpec(TaskSpec):
     def get_on_error(self):
         return self._on_error
 
+    def get_on_skip(self):
+        return self._on_skip
+
 
 class ReverseWorkflowTaskSpec(TaskSpec):
     _polymorphic_value = 'reverse'
@@ -384,7 +407,7 @@ class ReverseWorkflowTaskSpec(TaskSpec):
         self._requires = data.get('requires', [])
 
     def get_requires(self):
-        if isinstance(self._requires, six.string_types):
+        if isinstance(self._requires, str):
             return [self._requires]
 
         return self._requires
