@@ -93,34 +93,6 @@ class BaseLoggingCheckTest(base.BaseTest):
         self._assert_has_errors(code, checks.no_assert_true_false_is_not,
                                 expected_errors=errors)
 
-    def test_check_python3_xrange(self):
-        func = checks.check_python3_xrange
-        self.assertEqual(1, len(list(func('for i in xrange(10)'))))
-        self.assertEqual(1, len(list(func('for i in xrange    (10)'))))
-        self.assertEqual(0, len(list(func('for i in range(10)'))))
-        self.assertEqual(0, len(list(func('for i in range(10)'))))
-
-    def test_dict_iteritems(self):
-        self.assertEqual(1, len(list(checks.check_python3_no_iteritems(
-            "obj.iteritems()"))))
-
-        self.assertEqual(0, len(list(checks.check_python3_no_iteritems(
-            "six.iteritems(ob))"))))
-
-    def test_dict_iterkeys(self):
-        self.assertEqual(1, len(list(checks.check_python3_no_iterkeys(
-            "obj.iterkeys()"))))
-
-        self.assertEqual(0, len(list(checks.check_python3_no_iterkeys(
-            "six.iterkeys(ob))"))))
-
-    def test_dict_itervalues(self):
-        self.assertEqual(1, len(list(checks.check_python3_no_itervalues(
-            "obj.itervalues()"))))
-
-        self.assertEqual(0, len(list(checks.check_python3_no_itervalues(
-            "six.itervalues(ob))"))))
-
 
 class TestLoggingWithWarn(BaseLoggingCheckTest):
 
