@@ -1,4 +1,5 @@
 # Copyright 2017 - Brocade Communications Systems, Inc.
+# Copyright 2018 - Extreme Networks, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -12,24 +13,27 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
-import eventlet
+import threading
+import time
 
 from mistral.api import service as api_service
 from mistral.cmd import launch
+from mistral.scheduler import base as sched_base
 from mistral.tests.unit import base
 
 
 class ServiceLauncherTest(base.DbTestCase):
-
     def setUp(self):
         super(ServiceLauncherTest, self).setUp()
 
         self.override_config('enabled', False, group='cron_trigger')
 
         launch.reset_server_managers()
+        sched_base.destroy_system_scheduler()
 
     def test_launch_all(self):
-        eventlet.spawn(launch.launch_any, launch.LAUNCH_OPTIONS.keys())
+        threading.Thread(target=launch.launch_any,
+                         args=(launch.LAUNCH_OPTIONS.keys(),)).start()
 
         for i in range(0, 50):
             svr_proc_mgr = launch.get_server_process_manager()
@@ -38,7 +42,7 @@ class ServiceLauncherTest(base.DbTestCase):
             if svr_proc_mgr and svr_thrd_mgr:
                 break
 
-            eventlet.sleep(0.1)
+            time.sleep(0.1)
 
         self.assertIsNotNone(svr_proc_mgr)
         self.assertIsNotNone(svr_thrd_mgr)
@@ -47,10 +51,11 @@ class ServiceLauncherTest(base.DbTestCase):
         api_workers = api_server.workers
 
         self._await(lambda: len(svr_proc_mgr.children.keys()) == api_workers)
-        self._await(lambda: len(svr_thrd_mgr.services.services) == 3)
+        self._await(lambda: len(svr_thrd_mgr.services.services) == 4)
 
     def test_launch_process(self):
-        eventlet.spawn(launch.launch_any, ['api'])
+        threading.Thread(target=launch.launch_any,
+                         args=(['api'],)).start()
 
         for i in range(0, 50):
             svr_proc_mgr = launch.get_server_process_manager()
@@ -58,7 +63,7 @@ class ServiceLauncherTest(base.DbTestCase):
             if svr_proc_mgr:
                 break
 
-            eventlet.sleep(0.1)
+            time.sleep(0.1)
 
         svr_thrd_mgr = launch.get_server_thread_manager()
 
@@ -71,7 +76,8 @@ class ServiceLauncherTest(base.DbTestCase):
         self._await(lambda: len(svr_proc_mgr.children.keys()) == api_workers)
 
     def test_launch_thread(self):
-        eventlet.spawn(launch.launch_any, ['engine'])
+        threading.Thread(target=launch.launch_any,
+                         args=(['engine'],)).start()
 
         for i in range(0, 50):
             svr_thrd_mgr = launch.get_server_thread_manager()
@@ -79,7 +85,7 @@ class ServiceLauncherTest(base.DbTestCase):
             if svr_thrd_mgr:
                 break
 
-            eventlet.sleep(0.1)
+            time.sleep(0.1)
 
         svr_proc_mgr = launch.get_server_process_manager()
 
