@@ -16,13 +16,36 @@
 """Valid task and workflow states."""
 
 IDLE = 'IDLE'
+"""Task is not started yet."""
+
 WAITING = 'WAITING'
+"""
+Task execution object has been created, but it is not ready to start because
+some preconditions are not met.
+NOTE:
+The task may never run just because some of the preconditions may never be met.
+"""
+
 RUNNING = 'RUNNING'
+"""Task, action or workflow is currently being executed."""
+
 RUNNING_DELAYED = 'DELAYED'
+"""Task is in the running state but temporarily delayed."""
+
 PAUSED = 'PAUSED'
+"""Task, action or workflow has been paused."""
+
 SUCCESS = 'SUCCESS'
+"""Task, action or workflow has finished successfully."""
+
 CANCELLED = 'CANCELLED'
+"""Task, action or workflow has been cancelled."""
+
 ERROR = 'ERROR'
+"""Task, action or workflow has finished with an error."""
+
+SKIPPED = 'SKIPPED'
+"""Task has been skipped."""
 
 _ALL = [
     IDLE,
@@ -32,7 +55,8 @@ _ALL = [
     PAUSED,
     SUCCESS,
     CANCELLED,
-    ERROR
+    ERROR,
+    SKIPPED
 ]
 
 _VALID_TRANSITIONS = {
@@ -43,9 +67,11 @@ _VALID_TRANSITIONS = {
     PAUSED: [RUNNING, ERROR, CANCELLED],
     SUCCESS: [],
     CANCELLED: [RUNNING],
-    ERROR: [RUNNING]
+    ERROR: [RUNNING, SKIPPED]
 }
 
+TERMINAL_STATES = {SUCCESS, ERROR, CANCELLED}
+
 
 def is_valid(state):
     return state in _ALL
@@ -56,13 +82,17 @@ def is_invalid(state):
 
 
 def is_completed(state):
-    return state in [SUCCESS, ERROR, CANCELLED]
+    return state in [SUCCESS, ERROR, CANCELLED, SKIPPED]
 
 
 def is_cancelled(state):
     return state == CANCELLED
 
 
+def is_skipped(state):
+    return state == SKIPPED
+
+
 def is_running(state):
     return state in [RUNNING, RUNNING_DELAYED]
 
@@ -83,6 +113,10 @@ def is_paused_or_completed(state):
     return is_paused(state) or is_completed(state)
 
 
+def is_cancelled_or_skipped(state):
+    return is_cancelled(state) or is_skipped(state)
+
+
 def is_paused_or_idle(state):
     return is_paused(state) or is_idle(state)
 
